@@ -49,13 +49,13 @@ class Job::Private : public QThread
     Q_OBJECT
 
 public:
-    Private(Job *job, QObject *parent = 0)
+    Private(Job *job, QObject *parent = nullptr)
         : QThread(parent)
         , q(job)
     {
     }
 
-    virtual void run() Q_DECL_OVERRIDE;
+    void run() override;
 
 private:
     Job *q;
@@ -76,18 +76,15 @@ Job::Job(Archive *archive, ReadOnlyArchiveInterface *interface)
 }
 
 Job::Job(Archive *archive)
-    : Job(archive, Q_NULLPTR)
+    : Job(archive, nullptr)
 {}
 
 Job::Job(ReadOnlyArchiveInterface *interface)
-    : Job(Q_NULLPTR, interface)
+    : Job(nullptr, interface)
 {}
 
 Job::~Job()
 {
-    qDeleteAll(m_archiveEntries);
-    m_archiveEntries.clear();
-
     if (d->isRunning()) {
         d->wait();
     }
@@ -249,11 +246,11 @@ LoadJob::LoadJob(Archive *archive, ReadOnlyArchiveInterface *interface)
 }
 
 LoadJob::LoadJob(Archive *archive)
-    : LoadJob(archive, Q_NULLPTR)
+    : LoadJob(archive, nullptr)
 {}
 
 LoadJob::LoadJob(ReadOnlyArchiveInterface *interface)
-    : LoadJob(Q_NULLPTR, interface)
+    : LoadJob(nullptr, interface)
 {}
 
 void LoadJob::doWork()
@@ -574,7 +571,8 @@ QTemporaryDir *TempExtractJob::tempDir() const
 
 void TempExtractJob::doWork()
 {
-    emit description(this, i18n("Extracting one file"));
+    // pass 1 to i18np on purpose so this translation may properly be reused.
+    emit description(this, i18np("Extracting one file", "Extracting %1 files", 1));
 
     connectToArchiveInterfaceSignals();
 
@@ -631,7 +629,7 @@ void AddJob::doWork()
     }
 
     // Count total number of entries to be added.
-    qulonglong totalCount = 0;
+    uint totalCount = 0;
     QElapsedTimer timer;
     timer.start();
     foreach (const Archive::Entry* entry, m_entries) {
@@ -647,7 +645,7 @@ void AddJob::doWork()
 
     qCDebug(ARK) << "AddJob: going to add" << totalCount << "entries, counted in" << timer.elapsed() << "ms";
 
-    QString desc = i18np("Adding a file", "Adding %1 files", totalCount);
+    const QString desc = i18np("Compressing a file", "Compressing %1 files", totalCount);
     emit description(this, desc, qMakePair(i18n("Archive"), archiveInterface()->filename()));
 
     ReadWriteArchiveInterface *m_writeInterface =
