@@ -60,7 +60,6 @@
 #include <KToggleAction>
 #include <KLocalizedString>
 #include <KXMLGUIFactory>
-#include <kparts_version.h>
 
 #include <QAction>
 #include <QCursor>
@@ -203,7 +202,7 @@ Part::Part(QWidget *parentWidget, QObject *parent, const QVariantList& args)
     connect(m_model, &ArchiveModel::loadingFinished,
             this, &Part::slotLoadingFinished);
     connect(m_model, &ArchiveModel::droppedFiles,
-            this, static_cast<void (Part::*)(const QStringList&, const Archive::Entry*, const QString&)>(&Part::slotAddFiles));
+            this, QOverload<const QStringList&, const Archive::Entry*, const QString&>::of(&Part::slotAddFiles));
     connect(m_model, &ArchiveModel::error,
             this, &Part::slotError);
     connect(m_model, &ArchiveModel::messageWidget,
@@ -213,8 +212,12 @@ Part::Part(QWidget *parentWidget, QObject *parent, const QVariantList& args)
             this, &Part::setBusyGui);
     connect(this, &Part::ready,
             this, &Part::setReadyGui);
-    connect(this, static_cast<void (KParts::ReadOnlyPart::*)()>(&KParts::ReadOnlyPart::completed),
+    connect(this, &KParts::ReadOnlyPart::urlChanged,
             this, &Part::setFileNameFromArchive);
+    connect(this, QOverload<>::of(&KParts::ReadOnlyPart::completed),
+            this, &Part::setFileNameFromArchive);
+    connect(this, QOverload<>::of(&KParts::ReadOnlyPart::completed),
+            this, &Part::slotCompleted);
     connect(ArkSettings::self(), &KCoreConfigSkeleton::configChanged, this, &Part::updateActions);
 
     m_statusBarExtension = new KParts::StatusBarExtension(this);
@@ -258,6 +261,8 @@ void Part::registerJob(KJob* job)
         m_statusBarExtension->addStatusBarItem(m_jobTracker->widget(nullptr), 0, true);
         m_jobTracker->widget(job)->show();
     }
+
+    KIO::getJobTracker()->registerJob(job);
     m_jobTracker->registerJob(job);
 
     emit busy();
@@ -358,20 +363,21 @@ void Part::setupActions()
     connect(m_showInfoPanelAction, &QAction::triggered,
             this, &Part::slotToggleInfoPanel);
 
-    m_saveAsAction = actionCollection()->addAction(KStandardAction::SaveAs, QStringLiteral("ark_file_save_as"), this, SLOT(slotSaveAs()));
+    m_saveAsAction = KStandardAction::saveAs(this, &Part::slotSaveAs, nullptr);
+    actionCollection()->addAction(QStringLiteral("ark_file_save_as"), m_saveAsAction);
 
     m_openFileAction = actionCollection()->addAction(QStringLiteral("openfile"));
     m_openFileAction->setText(i18nc("open a file with external program", "&Open"));
     m_openFileAction->setIcon(QIcon::fromTheme(QStringLiteral("document-open")));
     m_openFileAction->setToolTip(i18nc("@info:tooltip", "Click to open the selected file with the associated application"));
-    connect(m_openFileAction, &QAction::triggered, m_signalMapper, static_cast<void (QSignalMapper::*)()>(&QSignalMapper::map));
+    connect(m_openFileAction, &QAction::triggered, m_signalMapper, QOverload<>::of(&QSignalMapper::map));
     m_signalMapper->setMapping(m_openFileAction, OpenFile);
 
     m_openFileWithAction = actionCollection()->addAction(QStringLiteral("openfilewith"));
     m_openFileWithAction->setText(i18nc("open a file with external program", "Open &With..."));
     m_openFileWithAction->setIcon(QIcon::fromTheme(QStringLiteral("document-open")));
     m_openFileWithAction->setToolTip(i18nc("@info:tooltip", "Click to open the selected file with an external program"));
-    connect(m_openFileWithAction, &QAction::triggered, m_signalMapper, static_cast<void (QSignalMapper::*)()>(&QSignalMapper::map));
+    connect(m_openFileWithAction, &QAction::triggered, m_signalMapper, QOverload<>::of(&QSignalMapper::map));
     m_signalMapper->setMapping(m_openFileWithAction, OpenFileWith);
 
     m_previewAction = actionCollection()->addAction(QStringLiteral("preview"));
@@ -379,7 +385,7 @@ void Part::setupActions()
     m_previewAction->setIcon(QIcon::fromTheme(QStringLiteral("document-preview-archive")));
     m_previewAction->setToolTip(i18nc("@info:tooltip", "Click to preview the selected file"));
     actionCollection()->setDefaultShortcut(m_previewAction, Qt::CTRL + Qt::Key_P);
-    connect(m_previewAction, &QAction::triggered, m_signalMapper, static_cast<void (QSignalMapper::*)()>(&QSignalMapper::map));
+    connect(m_previewAction, &QAction::triggered, m_signalMapper, QOverload<>::of(&QSignalMapper::map));
     m_signalMapper->setMapping(m_previewAction, Preview);
 
     m_extractArchiveAction = actionCollection()->addAction(QStringLiteral("extract_all"));
@@ -401,42 +407,17 @@ void Part::setupActions()
     m_addFilesAction->setText(i18n("Add &Files..."));
     m_addFilesAction->setToolTip(i18nc("@info:tooltip", "Click to add files to the archive"));
     actionCollection()->setDefaultShortcut(m_addFilesAction, Qt::ALT + Qt::Key_A);
-    connect(m_addFilesAction, &QAction::triggered, this, static_cast<void (Part::*)()>(&Part::slotAddFiles));
+    connect(m_addFilesAction, &QAction::triggered, this, QOverload<>::of(&Part::slotAddFiles));
 
-    m_renameFileAction = actionCollection()->addAction(QStringLiteral("rename"));
-    m_renameFileAction->setIcon(QIcon::fromTheme(QStringLiteral("edit-rename")));
-    m_renameFileAction->setText(i18n("&Rename"));
-    actionCollection()->setDefaultShortcut(m_renameFileAction, Qt::Key_F2);
-    m_renameFileAction->setToolTip(i18nc("@info:tooltip", "Click to rename the selected file"));
-    connect(m_renameFileAction, &QAction::triggered, m_view, &ArchiveView::renameSelectedEntry);
+    m_renameFileAction = KStandardAction::renameFile(m_view, &ArchiveView::renameSelectedEntry, actionCollection());
 
-    m_deleteFilesAction = actionCollection()->addAction(QStringLiteral("delete"));
+    m_deleteFilesAction = KStandardAction::deleteFile(this, &Part::slotDeleteFiles, actionCollection());
     m_deleteFilesAction->setIcon(QIcon::fromTheme(QStringLiteral("archive-remove")));
-    m_deleteFilesAction->setText(i18n("De&lete"));
     actionCollection()->setDefaultShortcut(m_deleteFilesAction, Qt::Key_Delete);
-    m_deleteFilesAction->setToolTip(i18nc("@info:tooltip", "Click to delete the selected files"));
-    connect(m_deleteFilesAction, &QAction::triggered, this, &Part::slotDeleteFiles);
 
-    m_cutFilesAction = actionCollection()->addAction(QStringLiteral("cut"));
-    m_cutFilesAction->setIcon(QIcon::fromTheme(QStringLiteral("edit-cut")));
-    m_cutFilesAction->setText(i18nc("@action:inmenu", "C&ut"));
-    actionCollection()->setDefaultShortcut(m_cutFilesAction, Qt::CTRL + Qt::Key_X);
-    m_cutFilesAction->setToolTip(i18nc("@info:tooltip", "Click to cut the selected files"));
-    connect(m_cutFilesAction, &QAction::triggered, this, &Part::slotCutFiles);
-
-    m_copyFilesAction = actionCollection()->addAction(QStringLiteral("copy"));
-    m_copyFilesAction->setIcon(QIcon::fromTheme(QStringLiteral("edit-copy")));
-    m_copyFilesAction->setText(i18nc("@action:inmenu", "C&opy"));
-    actionCollection()->setDefaultShortcut(m_copyFilesAction, Qt::CTRL + Qt::Key_C);
-    m_copyFilesAction->setToolTip(i18nc("@info:tooltip", "Click to copy the selected files"));
-    connect(m_copyFilesAction, &QAction::triggered, this, &Part::slotCopyFiles);
-
-    m_pasteFilesAction = actionCollection()->addAction(QStringLiteral("paste"));
-    m_pasteFilesAction->setIcon(QIcon::fromTheme(QStringLiteral("edit-paste")));
-    m_pasteFilesAction->setText(i18nc("@action:inmenu", "Pa&ste"));
-    actionCollection()->setDefaultShortcut(m_pasteFilesAction, Qt::CTRL + Qt::Key_V);
-    m_pasteFilesAction->setToolTip(i18nc("@info:tooltip", "Click to paste the files here"));
-    connect(m_pasteFilesAction, &QAction::triggered, this, static_cast<void (Part::*)()>(&Part::slotPasteFiles));
+    m_cutFilesAction = KStandardAction::cut(this, &Part::slotCutFiles, actionCollection());
+    m_copyFilesAction = KStandardAction::copy(this, &Part::slotCopyFiles, actionCollection());
+    m_pasteFilesAction = KStandardAction::paste(this, QOverload<>::of(&Part::slotPasteFiles), actionCollection());
 
     m_propertiesAction = actionCollection()->addAction(QStringLiteral("properties"));
     m_propertiesAction->setIcon(QIcon::fromTheme(QStringLiteral("document-properties")));
@@ -458,14 +439,9 @@ void Part::setupActions()
     m_testArchiveAction->setToolTip(i18nc("@info:tooltip", "Click to test the archive for integrity"));
     connect(m_testArchiveAction, &QAction::triggered, this, &Part::slotTestArchive);
 
-    m_searchAction = actionCollection()->addAction(QStringLiteral("find_in_archive"));
-    m_searchAction->setIcon(QIcon::fromTheme(QStringLiteral("search")));
-    m_searchAction->setText(i18nc("@action:inmenu", "&Find Files"));
-    actionCollection()->setDefaultShortcut(m_searchAction, Qt::CTRL + Qt::Key_F);
-    m_searchAction->setToolTip(i18nc("@info:tooltip", "Click to search in archive"));
-    connect(m_searchAction, &QAction::triggered, this, &Part::slotShowFind);
+    m_searchAction = KStandardAction::find(this, &Part::slotShowFind, actionCollection());
 
-    connect(m_signalMapper, static_cast<void (QSignalMapper::*)(int)>(&QSignalMapper::mapped),
+    connect(m_signalMapper, QOverload<int>::of(&QSignalMapper::mapped),
             this, &Part::slotOpenEntry);
 
     updateActions();
@@ -612,6 +588,11 @@ bool Part::isArchiveWritable() const
     return isReadWrite() && m_model->archive() && !m_model->archive()->isReadOnly();
 }
 
+bool Part::isCreatingNewArchive() const
+{
+    return arguments().metaData()[QStringLiteral("createNewArchive")] == QLatin1String("true");
+}
+
 void Part::createArchive()
 {
     const QString fixedMimeType = arguments().metaData()[QStringLiteral("fixedMimeType")];
@@ -626,9 +607,6 @@ void Part::createArchive()
         m_model->encryptArchive(password,
                                 arguments().metaData()[QStringLiteral("encryptHeader")] == QLatin1String("true"));
     }
-
-    updateActions();
-    m_view->setDropsEnabled(true);
 }
 
 void Part::loadArchive()
@@ -773,9 +751,7 @@ bool Part::openFile()
         return false;
     }
 
-    const bool creatingNewArchive = arguments().metaData()[QStringLiteral("createNewArchive")] == QLatin1String("true");
-
-    if (creatingNewArchive) {
+    if (isCreatingNewArchive()) {
         createArchive();
         return true;
     }
@@ -815,7 +791,6 @@ bool Part::isLocalFileValid()
 {
     const QString localFile = localFilePath();
     const QFileInfo localFileInfo(localFile);
-    const bool creatingNewArchive = arguments().metaData()[QStringLiteral("createNewArchive")] == QLatin1String("true");
 
     if (localFileInfo.isDir()) {
         displayMsgWidget(KMessageWidget::Error, xi18nc("@info",
@@ -824,7 +799,7 @@ bool Part::isLocalFileValid()
         return false;
     }
 
-    if (creatingNewArchive) {
+    if (isCreatingNewArchive()) {
         if (localFileInfo.exists()) {
             if (!confirmAndDelete(localFile)) {
                 displayMsgWidget(KMessageWidget::Error, xi18nc("@info",
@@ -870,53 +845,20 @@ bool Part::confirmAndDelete(const QString &targetFile)
     return QFile(targetFile).remove();
 }
 
-void Part::slotLoadingStarted()
+void Part::slotCompleted()
 {
-    m_model->filesToMove.clear();
-    m_model->filesToCopy.clear();
-}
-
-void Part::slotLoadingFinished(KJob *job)
-{
-    if (job->error()) {
-        if (arguments().metaData()[QStringLiteral("createNewArchive")] != QLatin1String("true")) {
-            if (job->error() != KJob::KilledJobError) {
-                displayMsgWidget(KMessageWidget::Error, xi18nc("@info", "Loading the archive <filename>%1</filename> failed with the following error:<nl/><message>%2</message>",
-                                                               localFilePath(),
-                                                               job->errorString()));
-            }
-
-            // The file failed to open, so reset the open archive, info panel and caption.
-            m_model->reset();
-            m_infoPanel->setPrettyFileName(QString());
-            m_infoPanel->updateWithDefaults();
-
-            emit setWindowCaption(QString());
-#if KPARTS_VERSION >= QT_VERSION_CHECK(5, 37, 0)
-            // See https://phabricator.kde.org/D6856
-            closeUrl();
-#else
-            setUrl(QUrl());
-#endif
-        }
-    } else {
-        emit completed();
-    }
-
-    m_view->sortByColumn(0, Qt::AscendingOrder);
-    m_view->expandIfSingleFolder();
-
-    // After loading all files, resize the columns to fit all fields
-    m_view->header()->resizeSections(QHeaderView::ResizeToContents);
-    // Now we can start accepting drops in the archive view (if loading was successful and the archive can be modified).
-    m_view->setDropsEnabled(!job->error() && isArchiveWritable());
-
-    updateActions();
-
-    if (!m_model->archive()) {
+    if (isCreatingNewArchive()) {
+        m_view->setDropsEnabled(true);
+        updateActions();
         return;
     }
 
+    // Existing archive, setup the view for it.
+    m_view->sortByColumn(0, Qt::AscendingOrder);
+    m_view->expandIfSingleFolder();
+    m_view->header()->resizeSections(QHeaderView::ResizeToContents);
+    m_view->setDropsEnabled(isArchiveWritable());
+
     if (!m_model->archive()->comment().isEmpty()) {
         m_commentView->setPlainText(m_model->archive()->comment());
         slotShowComment();
@@ -939,6 +881,35 @@ void Part::slotLoadingFinished(KJob *job)
     if (arguments().metaData()[QStringLiteral("showExtractDialog")] == QLatin1String("true")) {
         QTimer::singleShot(0, this, &Part::slotShowExtractionDialog);
     }
+
+    updateActions();
+}
+
+void Part::slotLoadingStarted()
+{
+    m_model->filesToMove.clear();
+    m_model->filesToCopy.clear();
+}
+
+void Part::slotLoadingFinished(KJob *job)
+{
+    if (!job->error()) {
+        emit completed();
+        return;
+    }
+
+    // Loading failed or was canceled by the user (e.g. password dialog rejected).
+    emit canceled(job->errorString());
+    m_view->setDropsEnabled(false);
+    m_model->reset();
+    closeUrl();
+    setFileNameFromArchive();
+    updateActions();
+    if (job->error() != KJob::KilledJobError) {
+        displayMsgWidget(KMessageWidget::Error, xi18nc("@info", "Loading the archive <filename>%1</filename> failed with the following error:<nl/><message>%2</message>",
+                                                       localFilePath(),
+                                                       job->errorString()));
+    }
 }
 
 void Part::setReadyGui()
